% Tables for MC results

clear all;
small = 1.0e-10;
big = 1.0e+8;
global datadir

% -- File Directories   
outdir = 'out/';
figdir = 'fig/';
matdir = 'mat/';
datdir = '../um_mc/';
addpath('../matlab_functions');

% List Designs
Design_lst = {'Levy';'I1001';'I1003';'I1mat';'LTU003';'LTU500';'Bsheet'};
k_lst = {'k1';'k5'};
est_lst = {'isodiff';'FGLS';'lfreg';'lowpass';'clustFE';'IM'}; 
% Note: lfreg is low-pass eigenvector; lowpass is high-pass eigenvector
pct_lst = {'5';'50';'95'};
n_design = length(Design_lst);
isodiff = NaN(2,3,2,n_design);
fgls = NaN(3,3,2,n_design);
clust = NaN(3,3,2,n_design);
IM = NaN(1,3,2,n_design);
lp_eig = NaN(1,3,2,n_design);
hp_eig = NaN(1,3,2,n_design);
r2 = NaN(3,n_design);

for ik = 1:2

if ik == 1
 kstr = 'k1';
elseif ik ==2
 kstr = 'k5';   
end


for id = 1:n_design
        dstr = char(Design_lst(id));
        isodiff(:,:,:,id) = choose_cols('isodiff',2,datdir,dstr,kstr);
        fgls(:,:,:,id) = choose_cols('FGLS',3,datdir,dstr,kstr);
        IM(:,:,:,id) = choose_cols('IM',1,datdir,dstr,kstr);
        lp_eig(:,:,:,id) = choose_cols('lfreg',1,datdir,dstr,kstr);
        hp_eig(:,:,:,id) = choose_cols('lowpass',1,datdir,dstr,kstr);
        clust(:,:,:,id) = choose_cols('clustFE',3,datdir,dstr,kstr); 
        scpc(:,:,:,id) = choose_cols('SCPC',1,datdir,dstr,kstr); 
end
 
for id = 1:n_design
    dstr = char(Design_lst(id));
    fname = [datdir dstr '_' kstr '_R2_50']; 
    tmp = readmatrix(fname);
    r2(2,id) = tmp;
    fname = [datdir dstr '_' kstr '_R2_5'];  
    tmp = readmatrix(fname);
    r2(1,id) = tmp;
    fname = [datdir dstr '_' kstr '_R2_95']; 
    tmp = readmatrix(fname);
    r2(2,id) = tmp;        
end

% Create Table
outfile_name = [outdir 'um_mc_tab_size_' kstr '.csv'];
fileID = fopen(outfile_name,'w');
fprintf(fileID,'Method, ');
for id = 1:n_design
        dstr = char(Design_lst(id));
        fprintf(fileID,dstr);
        if id == n_design
            fprintf(fileID,' \n');
        else
            fprintf(fileID,' ,');
        end
end

fprintf(fileID,'SCPC,');
tmp = squeeze(scpc(1,2,1,:));
prt_tmp(tmp,fileID);

fprintf(fileID,'isodiff (CSCPC),');
tmp = squeeze(isodiff(2,2,1,:));
prt_tmp(tmp,fileID);

fprintf(fileID,'clust (Cluster),');
tmp = squeeze(clust(1,2,1,:));
prt_tmp(tmp,fileID);

fprintf(fileID,'clust (CSCPC),');
tmp = squeeze(clust(3,2,1,:));
prt_tmp(tmp,fileID);

fprintf(fileID,'FGLS,');
tmp = squeeze(fgls(1,2,1,:));
prt_tmp(tmp,fileID);

fprintf(fileID,'FGLS (CSCPC),');
tmp = squeeze(fgls(3,2,1,:));
prt_tmp(tmp,fileID);

fprintf(fileID,'lp_eig,');
tmp = squeeze(lp_eig(1,2,1,:));
prt_tmp(tmp,fileID);

fprintf(fileID,'hp_eig,');
tmp = squeeze(hp_eig(1,2,1,:));
prt_tmp(tmp,fileID);

fprintf(fileID,'IM,');
tmp = squeeze(IM(1,2,1,:));
prt_tmp(tmp,fileID);

fprintf(fileID,'R2,');
tmp = r2(2,:);
prt_tmp(tmp,fileID);

% Create Table
outfile_name = [outdir 'um_mc_tab_length_' kstr '.csv'];
fileID = fopen(outfile_name,'w');
fprintf(fileID,'Method, ');
for id = 1:n_design
        dstr = char(Design_lst(id));
        fprintf(fileID,dstr);
        if id == n_design
            fprintf(fileID,' \n');
        else
            fprintf(fileID,' ,');
        end
end

% fprintf(fileID,'SCPC,');
% tmp = squeeze(scpc(1,2,2,:));
% prt_tmp(tmp,fileID);

fprintf(fileID,'isodiff (CSCPC),');
tmp = squeeze(isodiff(2,2,2,:));
prt_tmp(tmp,fileID);

fprintf(fileID,'clust (CSCPC),');
tmp = squeeze(clust(3,2,2,:));
prt_tmp(tmp,fileID);

% fprintf(fileID,'FGLS,');
% tmp = squeeze(fgls(1,2,2,:));
% prt_tmp(tmp,fileID);

fprintf(fileID,'FGLS (CSCPC),');
tmp = squeeze(fgls(3,2,2,:));
prt_tmp(tmp,fileID);

fprintf(fileID,'lp_eig,');
tmp = squeeze(lp_eig(1,2,2,:));
prt_tmp(tmp,fileID);

fprintf(fileID,'hp_eig,');
tmp = squeeze(hp_eig(1,2,2,:));
prt_tmp(tmp,fileID);

fprintf(fileID,'IM,');
tmp = squeeze(IM(1,2,2,:));
prt_tmp(tmp,fileID);

end


% Functions
function [rslt] = choose_cols(estr,ic,datdir,dstr,kstr)
 % Get Results 50th percentile
        rslt = NaN(ic,3,2);
        for ii = 1:ic
            irow = 1+2*(ii-1);
            fname = [datdir dstr '_' kstr '_' estr '_50']; 
            tmp = readmatrix(fname);
            % Eliminate last entry for lowpass (= hp_eig)
            if strcmp(estr,'lowpass') == 1
                tmp = tmp(:,1:end-1);
                if size(tmp,2) ~= 4
                    error('size is wrong')
                end
            end
            tmp1 = tmp(irow,:)';
            [~,i1]=min(abs(tmp1-0.05));
            rslt(ii,2,1)=tmp(irow,i1);
            rslt(ii,2,2)=tmp(irow+1,i1);
            fname = [datdir dstr '_' kstr '_' estr '_5']; 
            tmp = readmatrix(fname);
            rslt(ii,1,1)=tmp(irow,i1);
            rslt(ii,1,2)=tmp(irow+1,i1);
            fname = [datdir dstr '_' kstr '_' estr '_95']; 
            tmp = readmatrix(fname);
            rslt(ii,3,1)=tmp(irow,i1);
            rslt(ii,3,2)=tmp(irow+1,i1);            
        end
end

function prt_tmp(tmp,fileID)
n_design = length(tmp);
for id = 1:n_design
        fprintf(fileID,'%5.2f',tmp(id));
        if id == n_design
            fprintf(fileID,' \n');
        else
            fprintf(fileID,' ,');
        end
end

end

